"""This file contains the constants for the connexion module."""

# Ports and communication constants
DISCOVERY_PORT = 37020

# Communication protocol

HEADER = 'header'
NEW_ID = 'new_id'
ID = 'id'
EXIT = 'exit'
CONTENT = 'content'
BROADCAST_IP = 'broadcast_ip'
HOST_PORT = 'host_port'
TIMESTAMP = 'timestamp'
NEW_PHASE = 'new_phase'
DISCONNECTION = 'disconnection'

ONLINE = 'online'
OFFLINE = 'offline'
